@@ -1,30 +1,7 @@
 #####################################################################################
 #
 #  Copyright (c) Crossbar.io Technologies GmbH
-#
-#  Unless a separate license agreement exists between you and Crossbar.io GmbH (e.g.
-#  you have purchased a commercial license), the license terms below apply.
-#
-#  Should you enter into a separate license agreement after having received a copy of
-#  this software, then the terms of such license agreement replace the terms below at
-#  the time at which such license agreement becomes effective.
-#
-#  In case a separate license agreement ends, and such agreement ends without being
-#  replaced by another separate license agreement, the license terms below apply
-#  from the time at which said agreement ends.
-#
-#  LICENSE TERMS
-#
-#  This program is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License, version 3, as published by the
-#  Free Software Foundation. This program is distributed in the hope that it will be
-#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-#
-#  See the GNU Affero General Public License Version 3 for more details.
-#
-#  You should have received a copy of the GNU Affero General Public license along
-#  with this program. If not, see <http://www.gnu.org/licenses/agpl-3.0.en.html>.
+#  SPDX-License-Identifier: EUPL-1.2
 #
 #####################################################################################
 
@@ -39,14 +16,23 @@ txaio.use_twisted()
 from txaio import make_logger
 
 from .protocol import (
-    MQTTParser, Failure,
+    MQTTParser,
+    Failure,
 )
 from ._events import (
-    Connect, ConnACK,
-    Subscribe, SubACK,
-    Unsubscribe, UnsubACK,
-    Publish, PubACK, PubREC, PubREL, PubCOMP,
-    PingREQ, PingRESP,
+    Connect,
+    ConnACK,
+    Subscribe,
+    SubACK,
+    Unsubscribe,
+    UnsubACK,
+    Publish,
+    PubACK,
+    PubREC,
+    PubREL,
+    PubCOMP,
+    PingREQ,
+    PingRESP,
     Disconnect,
 )
 
@@ -148,8 +134,7 @@ class MQTTServerTwistedProtocol(Protocol):
         #                if the subscriptions are unsuccessful -- their
         #                unsuccessfulness is listed in the return codes
         # MQTT-3.8.4-2 - the suback needs to have the same packet id
-        suback = SubACK(packet_identifier=packet_identifier,
-                        return_codes=return_codes)
+        suback = SubACK(packet_identifier=packet_identifier, return_codes=return_codes)
         self._send_packet(suback)
 
     def send_publish(self, topic, qos, body, retained):
@@ -157,8 +142,7 @@ class MQTTServerTwistedProtocol(Protocol):
         if qos not in [0, 1, 2]:
             raise ValueError("QoS must be [0, 1, 2]")
 
-        self.session.queued_messages.append(
-            Message(topic=topic, qos=qos, body=body, retained=retained))
+        self.session.queued_messages.append(Message(topic=topic, qos=qos, body=body, retained=retained))
 
         if not self._flush_publishes and self._connected:
             self._flush_publishes = self._reactor.callLater(0, self._flush_saved_messages)
@@ -166,21 +150,30 @@ class MQTTServerTwistedProtocol(Protocol):
     def _send_publish(self, topic, qos, body, retained):
 
         if qos == 0:
-            publish = Publish(duplicate=False, qos_level=qos, retain=retained,
-                              packet_identifier=None, topic_name=topic,
+            publish = Publish(duplicate=False,
+                              qos_level=qos,
+                              retain=retained,
+                              packet_identifier=None,
+                              topic_name=topic,
                               payload=body)
 
         elif qos == 1:
             packet_id = self.session.get_packet_id()
-            publish = Publish(duplicate=False, qos_level=qos, retain=retained,
-                              packet_identifier=packet_id, topic_name=topic,
+            publish = Publish(duplicate=False,
+                              qos_level=qos,
+                              retain=retained,
+                              packet_identifier=packet_id,
+                              topic_name=topic,
                               payload=body)
 
         elif qos == 2:
 
             packet_id = self.session.get_packet_id()
-            publish = Publish(duplicate=False, qos_level=qos, retain=retained,
-                              packet_identifier=packet_id, topic_name=topic,
+            publish = Publish(duplicate=False,
+                              qos_level=qos,
+                              retain=retained,
+                              packet_identifier=packet_id,
+                              topic_name=topic,
                               payload=body)
 
         else:
@@ -190,7 +183,8 @@ class MQTTServerTwistedProtocol(Protocol):
         self._send_packet(publish)
 
     def _lose_connection(self):
-        self.log.debug(log_category="MQ400", client_id=self.session.client_id,
+        self.log.debug(log_category="MQ400",
+                       client_id=self.session.client_id,
                        seconds=self._timeout_time,
                        conn_id=self._connection_id)
         if self.transport:
@@ -199,8 +193,10 @@ class MQTTServerTwistedProtocol(Protocol):
             self.log.debug(log_category="MQ404")
 
     def _send_packet(self, packet):
-        self.log.trace(log_category="MQ101", client_id=self.session.client_id,
-                       packet=packet, conn_id=self._connection_id)
+        self.log.trace(log_category="MQ101",
+                       client_id=self.session.client_id,
+                       packet=packet,
+                       conn_id=self._connection_id)
         self.transport.write(packet.serialise())
 
     def _flush_saved_messages(self):
@@ -215,8 +211,7 @@ class MQTTServerTwistedProtocol(Protocol):
         # New, queued messages
         while self.session.queued_messages:
             message = self.session.queued_messages.popleft()
-            self._send_publish(message.topic, message.qos, message.body,
-                               message.retained)
+            self._send_publish(message.topic, message.qos, message.body, message.retained)
 
     @inlineCallbacks
     def _handle(self, data):
@@ -242,8 +237,10 @@ class MQTTServerTwistedProtocol(Protocol):
     def _handle_events(self, events):
 
         for event in events:
-            self.log.trace(log_category="MQ100", conn_id=self._connection_id,
-                           client_id=self.session.client_id, packet=event)
+            self.log.trace(log_category="MQ100",
+                           conn_id=self._connection_id,
+                           client_id=self.session.client_id,
+                           packet=event)
 
             if isinstance(event, Connect):
                 try:
@@ -262,8 +259,7 @@ class MQTTServerTwistedProtocol(Protocol):
                     # alive time.
                     if event.keep_alive:
                         self._timeout_time = event.keep_alive * 1.5
-                        self._timeout = self._reactor.callLater(
-                            self._timeout_time, self._lose_connection)
+                        self._timeout = self._reactor.callLater(self._timeout_time, self._lose_connection)
 
                     self.session.client_id = event.client_id
 
@@ -278,8 +274,7 @@ class MQTTServerTwistedProtocol(Protocol):
                     self.transport.loseConnection()
                     returnValue(None)
 
-                connack = ConnACK(session_present=session_present,
-                                  return_code=accept_conn)
+                connack = ConnACK(session_present=session_present, return_code=accept_conn)
                 self._send_packet(connack)
 
                 if accept_conn != 0:
@@ -299,8 +294,7 @@ class MQTTServerTwistedProtocol(Protocol):
                     # subscribing raising an exception), we must close the
                     # connection.
                     self.log.failure()
-                    self.log.failure(
-                        log_category="MQ501", client_id=self.session.client_id)
+                    self.log.failure(log_category="MQ501", client_id=self.session.client_id)
                     self.transport.loseConnection()
                     returnValue(None)
 
@@ -313,8 +307,7 @@ class MQTTServerTwistedProtocol(Protocol):
                     # MQTT-4.8.0-2 - If we get a transient error (like
                     # unsubscribing raising an exception), we must close the
                     # connection.
-                    self.log.failure(
-                        log_category="MQ502", client_id=self.session.client_id)
+                    self.log.failure(log_category="MQ502", client_id=self.session.client_id)
                     self.transport.loseConnection()
                     returnValue(None)
                 unsuback = UnsubACK(packet_identifier=event.packet_identifier)
@@ -330,13 +323,11 @@ class MQTTServerTwistedProtocol(Protocol):
                         # MQTT-4.8.0-2 - If we get a transient error (like
                         # publishing raising an exception), we must close the
                         # connection.
-                        self.log.failure(log_category="MQ503",
-                                         client_id=self.session.client_id)
+                        self.log.failure(log_category="MQ503", client_id=self.session.client_id)
                         self.transport.loseConnection()
                         returnValue(None)
 
-                    self.log.debug(log_category="MQ201", publish=event,
-                                   client_id=self.session.client_id)
+                    self.log.debug(log_category="MQ201", publish=event, client_id=self.session.client_id)
                     continue
 
                 elif event.qos_level == 1:
@@ -348,13 +339,11 @@ class MQTTServerTwistedProtocol(Protocol):
                         # publishing raising an exception), we must close the
                         # connection.
                         self.log.failure()
-                        self.log.failure(log_category="MQ504",
-                                         client_id=self.session.client_id)
+                        self.log.failure(log_category="MQ504", client_id=self.session.client_id)
                         self.transport.loseConnection()
                         returnValue(None)
 
-                    self.log.debug(log_category="MQ202", publish=event,
-                                   client_id=self.session.client_id)
+                    self.log.debug(log_category="MQ202", publish=event, client_id=self.session.client_id)
 
                     puback = PubACK(packet_identifier=event.packet_identifier)
                     self._send_packet(puback)
@@ -372,13 +361,11 @@ class MQTTServerTwistedProtocol(Protocol):
                         # MQTT-4.8.0-2 - If we get a transient error (like
                         # publishing raising an exception), we must close the
                         # connection.
-                        self.log.failure(log_category="MQ505",
-                                         client_id=self.session.client_id)
+                        self.log.failure(log_category="MQ505", client_id=self.session.client_id)
                         self.transport.loseConnection()
                         returnValue(None)
 
-                    self.log.debug(log_category="MQ203", publish=event,
-                                   client_id=self.session.client_id)
+                    self.log.debug(log_category="MQ203", publish=event, client_id=self.session.client_id)
 
                     pubrec = PubREC(packet_identifier=event.packet_identifier)
                     self._send_packet(pubrec)
@@ -390,8 +377,7 @@ class MQTTServerTwistedProtocol(Protocol):
                     # events implementation (it will be caught before it gets
                     # here), but the tests do some trickery to cover this
                     # case :)
-                    self.log.error(log_category="MQ403",
-                                   client_id=self.session.client_id)
+                    self.log.error(log_category="MQ403", client_id=self.session.client_id)
                     self.transport.loseConnection()
                     return
 
@@ -460,13 +446,11 @@ class MQTTServerTwistedProtocol(Protocol):
 
             else:
                 if isinstance(event, Failure):
-                    self.log.error(
-                        log_category="MQ401", client_id=self.session.client_id,
-                        error=event.reason)
+                    self.log.error(log_category="MQ401", client_id=self.session.client_id, error=event.reason)
                 else:
-                    self.log.error(
-                        log_category="MQ402", client_id=self.session.client_id,
-                        packet_id=event.__class__.__name__)
+                    self.log.error(log_category="MQ402",
+                                   client_id=self.session.client_id,
+                                   packet_id=event.__class__.__name__)
 
                 # Conformance statement MQTT-4.8.0-1: Must close the connection
                 # on a protocol violation.
