@@ -1,30 +1,7 @@
 #####################################################################################
 #
 #  Copyright (c) Crossbar.io Technologies GmbH
-#
-#  Unless a separate license agreement exists between you and Crossbar.io GmbH (e.g.
-#  you have purchased a commercial license), the license terms below apply.
-#
-#  Should you enter into a separate license agreement after having received a copy of
-#  this software, then the terms of such license agreement replace the terms below at
-#  the time at which such license agreement becomes effective.
-#
-#  In case a separate license agreement ends, and such agreement ends without being
-#  replaced by another separate license agreement, the license terms below apply
-#  from the time at which said agreement ends.
-#
-#  LICENSE TERMS
-#
-#  This program is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License, version 3, as published by the
-#  Free Software Foundation. This program is distributed in the hope that it will be
-#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-#
-#  See the GNU Affero General Public License Version 3 for more details.
-#
-#  You should have received a copy of the GNU Affero General Public license along
-#  with this program. If not, see <http://www.gnu.org/licenses/agpl-3.0.en.html>.
+#  SPDX-License-Identifier: EUPL-1.2
 #
 #####################################################################################
 
@@ -55,7 +32,6 @@ from cryptography.hazmat.primitives import hmac as hazmat_hmac
 from autobahn.websocket.utf8validator import Utf8Validator
 _validator = Utf8Validator()
 
-
 _ALLOWED_CONTENT_TYPES = set([b'application/json'])
 
 
@@ -155,8 +131,7 @@ class _CommonResource(Resource):
         self.log.debug(code=code, **kwargs)
 
         error_str = log_categories[kwargs['log_category']].format(**kwargs)
-        body = dump_json({"error": error_str,
-                          "args": [], "kwargs": {}}, True).encode('utf8')
+        body = dump_json({"error": error_str, "args": [], "kwargs": {}}, True).encode('utf8')
         request.setResponseCode(code)
         return body
 
@@ -238,8 +213,7 @@ class _CommonResource(Resource):
 
         try:
             if request.method not in (b"POST", b"PUT", b"OPTIONS"):
-                return self._deny_request(request, 405, method=request.method,
-                                          allowed="POST, PUT")
+                return self._deny_request(request, 405, method=request.method, allowed="POST, PUT")
             else:
 
                 if request.method == b"OPTIONS":
@@ -273,10 +247,7 @@ class _CommonResource(Resource):
         content_type_header = headers.getRawHeaders(b"content-type", [])
 
         if len(content_type_header) > 0:
-            content_type_elements = [
-                x.strip().lower()
-                for x in content_type_header[0].split(b";")
-            ]
+            content_type_elements = [x.strip().lower() for x in content_type_header[0].split(b";")]
         else:
             content_type_elements = []
 
@@ -286,12 +257,11 @@ class _CommonResource(Resource):
             # parsing anyway)
             if len(content_type_elements) > 0 and \
                content_type_elements[0] not in _ALLOWED_CONTENT_TYPES:
-                return self._deny_request(
-                    request, 400,
-                    accepted=list(_ALLOWED_CONTENT_TYPES),
-                    given=content_type_elements[0],
-                    log_category="AR452"
-                )
+                return self._deny_request(request,
+                                          400,
+                                          accepted=list(_ALLOWED_CONTENT_TYPES),
+                                          given=content_type_elements[0],
+                                          log_category="AR452")
 
         encoding_parts = {}
 
@@ -317,9 +287,7 @@ class _CommonResource(Resource):
         charset_encoding = encoding_parts.get("charset", "utf-8")
 
         if charset_encoding not in ["utf-8", 'utf8']:
-            return self._deny_request(
-                request, 400,
-                log_category="AR450")
+            return self._deny_request(request, 400, log_category="AR450")
 
         # enforce "post_body_limit"
         #
@@ -329,9 +297,7 @@ class _CommonResource(Resource):
         if len(content_length_header) == 1:
             content_length = int(content_length_header[0])
         elif len(content_length_header) > 1:
-            return self._deny_request(
-                request, 400,
-                log_category="AR463")
+            return self._deny_request(request, 400, log_category="AR463")
         else:
             content_length = body_length
 
@@ -340,16 +306,10 @@ class _CommonResource(Resource):
             # Content-Length. This is so that clients can't lie and bypass
             # length restrictions by giving an incorrect header with a large
             # body.
-            return self._deny_request(request, 400, bodylen=body_length,
-                                      conlen=content_length,
-                                      log_category="AR465")
+            return self._deny_request(request, 400, bodylen=body_length, conlen=content_length, log_category="AR465")
 
         if self._post_body_limit and content_length > self._post_body_limit:
-            return self._deny_request(
-                request, 413,
-                length=content_length,
-                accepted=self._post_body_limit
-            )
+            return self._deny_request(request, 413, length=content_length, accepted=self._post_body_limit)
 
         #
         # if we were given a GitHub token, check for a valid signature
@@ -360,7 +320,8 @@ class _CommonResource(Resource):
         if github_secret:
             if not _confirm_github_signature(request, github_secret, body):
                 return self._deny_request(
-                    request, 400,
+                    request,
+                    400,
                     log_cagegory="AR467",
                 )
 
@@ -374,10 +335,7 @@ class _CommonResource(Resource):
             key_str = args["key"]
         else:
             if self._secret:
-                return self._deny_request(
-                    request, 400,
-                    reason="'key' field missing",
-                    log_category="AR461")
+                return self._deny_request(request, 400, reason="'key' field missing", log_category="AR461")
 
         # timestamp
         #
@@ -387,19 +345,20 @@ class _CommonResource(Resource):
                 ts = datetime.datetime.strptime(native_string(timestamp_str), "%Y-%m-%dT%H:%M:%S.%fZ")
                 delta = abs((ts - datetime.datetime.utcnow()).total_seconds())
                 if self._timestamp_delta_limit and delta > self._timestamp_delta_limit:
-                    return self._deny_request(
-                        request, 400,
-                        log_category="AR464")
+                    return self._deny_request(request, 400, log_category="AR464")
             except ValueError:
                 return self._deny_request(
-                    request, 400,
-                    reason="invalid timestamp '{0}' (must be UTC/ISO-8601, e.g. '2011-10-14T16:59:51.123Z')".format(native_string(timestamp_str)),
+                    request,
+                    400,
+                    reason="invalid timestamp '{0}' (must be UTC/ISO-8601, e.g. '2011-10-14T16:59:51.123Z')".format(
+                        native_string(timestamp_str)),
                     log_category="AR462")
         else:
             if self._secret:
-                return self._deny_request(
-                    request, 400, reason="signed request required, but mandatory 'timestamp' field missing",
-                    log_category="AR461")
+                return self._deny_request(request,
+                                          400,
+                                          reason="signed request required, but mandatory 'timestamp' field missing",
+                                          log_category="AR461")
 
         # seq
         #
@@ -409,16 +368,14 @@ class _CommonResource(Resource):
                 # FIXME: check sequence
                 seq = int(seq_str)  # noqa
             except:
-                return self._deny_request(
-                    request, 400,
-                    reason="invalid sequence number '{0}' (must be an integer)".format(native_string(seq_str)),
-                    log_category="AR462")
+                return self._deny_request(request,
+                                          400,
+                                          reason="invalid sequence number '{0}' (must be an integer)".format(
+                                              native_string(seq_str)),
+                                          log_category="AR462")
         else:
             if self._secret:
-                return self._deny_request(
-                    request, 400,
-                    reason="'seq' field missing",
-                    log_category="AR461")
+                return self._deny_request(request, 400, reason="'seq' field missing", log_category="AR461")
 
         # nonce
         #
@@ -428,16 +385,14 @@ class _CommonResource(Resource):
                 # FIXME: check nonce
                 nonce = int(nonce_str)  # noqa
             except:
-                return self._deny_request(
-                    request, 400,
-                    reason="invalid nonce '{0}' (must be an integer)".format(native_string(nonce_str)),
-                    log_category="AR462")
+                return self._deny_request(request,
+                                          400,
+                                          reason="invalid nonce '{0}' (must be an integer)".format(
+                                              native_string(nonce_str)),
+                                          log_category="AR462")
         else:
             if self._secret:
-                return self._deny_request(
-                    request, 400,
-                    reason="'nonce' field missing",
-                    log_category="AR461")
+                return self._deny_request(request, 400, reason="'nonce' field missing", log_category="AR461")
 
         # signature
         #
@@ -445,20 +400,17 @@ class _CommonResource(Resource):
             signature_str = args["signature"]
         else:
             if self._secret:
-                return self._deny_request(
-                    request, 400,
-                    reason="'signature' field missing",
-                    log_category="AR461")
+                return self._deny_request(request, 400, reason="'signature' field missing", log_category="AR461")
 
         # do more checks if signed requests are required
         #
         if self._secret:
 
             if key_str != self._key:
-                return self._deny_request(
-                    request, 401,
-                    reason="unknown key '{0}' in signed request".format(native_string(key_str)),
-                    log_category="AR460")
+                return self._deny_request(request,
+                                          401,
+                                          reason="unknown key '{0}' in signed request".format(native_string(key_str)),
+                                          log_category="AR460")
 
             # Compute signature: HMAC[SHA256]_{secret} (key | timestamp | seq | nonce | body) => signature
             hm = hmac.new(self._secret, None, hashlib.sha256)
@@ -470,11 +422,9 @@ class _CommonResource(Resource):
             signature_recomputed = base64.urlsafe_b64encode(hm.digest())
 
             if signature_str != signature_recomputed:
-                return self._deny_request(request, 401,
-                                          log_category="AR459")
+                return self._deny_request(request, 401, log_category="AR459")
             else:
-                self.log.debug("REST request signature valid.",
-                               log_category="AR203")
+                self.log.debug("REST request signature valid.", log_category="AR203")
 
         # user_agent = headers.get("user-agent", "unknown")
         client_ip = request.getClientIP()
@@ -496,8 +446,7 @@ class _CommonResource(Resource):
         #
         if self._require_tls:
             if not is_secure:
-                return self._deny_request(request, 400,
-                                          reason="request denied because not using TLS")
+                return self._deny_request(request, 400, reason="request denied because not using TLS")
 
         # FIXME: authorize request
         authorized = True
@@ -511,9 +460,7 @@ class _CommonResource(Resource):
         # validate() returns a 4-tuple, of which item 0 is whether it
         # is valid
         if not validation_result[0]:
-            return self._deny_request(
-                request, 400,
-                log_category="AR451")
+            return self._deny_request(request, 400, log_category="AR451")
 
         event = body.decode('utf8')
 
@@ -521,14 +468,10 @@ class _CommonResource(Resource):
             try:
                 event = json.loads(event)
             except Exception as e:
-                return self._deny_request(
-                    request, 400,
-                    exc=e, log_category="AR453")
+                return self._deny_request(request, 400, exc=e, log_category="AR453")
 
             if not isinstance(event, dict):
-                return self._deny_request(
-                    request, 400,
-                    log_category="AR454")
+                return self._deny_request(request, 400, log_category="AR454")
 
         d = self._process(request, event)
 
