@@ -1,30 +1,7 @@
 #####################################################################################
 #
 #  Copyright (c) Crossbar.io Technologies GmbH
-#
-#  Unless a separate license agreement exists between you and Crossbar.io GmbH (e.g.
-#  you have purchased a commercial license), the license terms below apply.
-#
-#  Should you enter into a separate license agreement after having received a copy of
-#  this software, then the terms of such license agreement replace the terms below at
-#  the time at which such license agreement becomes effective.
-#
-#  In case a separate license agreement ends, and such agreement ends without being
-#  replaced by another separate license agreement, the license terms below apply
-#  from the time at which said agreement ends.
-#
-#  LICENSE TERMS
-#
-#  This program is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License, version 3, as published by the
-#  Free Software Foundation. This program is distributed in the hope that it will be
-#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-#
-#  See the GNU Affero General Public License Version 3 for more details.
-#
-#  You should have received a copy of the GNU Affero General Public license along
-#  with this program. If not, see <http://www.gnu.org/licenses/agpl-3.0.en.html>.
+#  SPDX-License-Identifier: EUPL-1.2
 #
 #####################################################################################
 
@@ -65,7 +42,6 @@ class TestBrokerPublish(unittest.TestCase):
     """
     Tests for crossbar.router.broker.Broker
     """
-
     def setUp(self):
         """
         Setup router and router session factories.
@@ -81,21 +57,18 @@ class TestBrokerPublish(unittest.TestCase):
         # allow everything
         self.router = self.router_factory.get('realm1')
         self.router.add_role(
-            RouterRoleStaticAuth(
-                self.router,
-                'test_role',
-                default_permissions={
-                    'uri': 'com.example.',
-                    'match': 'prefix',
-                    'allow': {
-                        'call': True,
-                        'register': True,
-                        'publish': True,
-                        'subscribe': True,
-                    }
-                }
-            )
-        )
+            RouterRoleStaticAuth(self.router,
+                                 'test_role',
+                                 default_permissions={
+                                     'uri': 'com.example.',
+                                     'match': 'prefix',
+                                     'allow': {
+                                         'call': True,
+                                         'register': True,
+                                         'publish': True,
+                                         'subscribe': True,
+                                     }
+                                 }))
 
         # create a router session factory
         self.session_factory = RouterSessionFactory(self.router_factory)
@@ -111,7 +84,6 @@ class TestBrokerPublish(unittest.TestCase):
         d = txaio.create_future()
 
         class TestSession(ApplicationSession):
-
             def onJoin(self, details):
                 txaio.resolve(d, None)
 
@@ -173,6 +145,7 @@ class TestBrokerPublish(unittest.TestCase):
 
         def boom(*args, **kw):
             raise the_exception
+
         session = self.session_factory()  # __call__ on the _RouterSessionFactory
         session.onHello = boom
         session.onOpen(transport)
@@ -205,6 +178,7 @@ class TestBrokerPublish(unittest.TestCase):
 
         def boom(*args, **kw):
             raise the_exception
+
         session = self.session_factory()  # __call__ on the _RouterSessionFactory
         session.onAuthenticate = boom
         session.onOpen(transport)
@@ -226,14 +200,12 @@ class TestBrokerPublish(unittest.TestCase):
         session = ApplicationSession()
         session.onLeave = mock.Mock()
         session._realm = 'realm'
-        router = Router(
-            factory=mock.Mock(),
-            realm=RouterRealm(
-                controller=MockContainer(),
-                id='realm',
-                config=dict(name='realm'),
-            )
-        )
+        router = Router(factory=mock.Mock(),
+                        realm=RouterRealm(
+                            controller=MockContainer(),
+                            id='realm',
+                            config=dict(name='realm'),
+                        ))
         rap = RouterApplicationSession(session, router)
 
         rap.send(message.Goodbye('wamp.reason.logout', 'some custom message'))
@@ -254,16 +226,15 @@ class TestBrokerPublish(unittest.TestCase):
 
         def boom(*args, **kw):
             raise the_exception
+
         session.onLeave = mock.Mock(side_effect=boom)
         session._realm = 'realm'
-        router = Router(
-            factory=mock.Mock(),
-            realm=RouterRealm(
-                controller=MockContainer(),
-                id='realm',
-                config=dict(name='realm'),
-            )
-        )
+        router = Router(factory=mock.Mock(),
+                        realm=RouterRealm(
+                            controller=MockContainer(),
+                            id='realm',
+                            config=dict(name='realm'),
+                        ))
         rap = RouterApplicationSession(session, router)
 
         rap.send(message.Goodbye('wamp.reason.logout', 'some custom message'))
@@ -285,16 +256,15 @@ class TestBrokerPublish(unittest.TestCase):
             if args[0] == 'disconnect':
                 return defer.fail(the_exception)
             return defer.succeed(None)
+
         session.fire = mock.Mock(side_effect=boom)
         session._realm = 'realm'
-        router = Router(
-            factory=mock.Mock(),
-            realm=RouterRealm(
-                controller=MockContainer(),
-                id='realm',
-                config=dict(name='realm'),
-            )
-        )
+        router = Router(factory=mock.Mock(),
+                        realm=RouterRealm(
+                            controller=MockContainer(),
+                            id='realm',
+                            config=dict(name='realm'),
+                        ))
         rap = RouterApplicationSession(session, router)
 
         rap.send(message.Goodbye('wamp.reason.logout', 'some custom message'))
@@ -318,14 +288,12 @@ class TestBrokerPublish(unittest.TestCase):
         session = ApplicationSession()
         session._realm = 'realm'
         session.fire = mock.Mock(side_effect=mock_fire)
-        router = Router(
-            factory=mock.Mock(),
-            realm=RouterRealm(
-                controller=MockContainer(),
-                id='realm',
-                config=dict(name='realm'),
-            )
-        )
+        router = Router(factory=mock.Mock(),
+                        realm=RouterRealm(
+                            controller=MockContainer(),
+                            id='realm',
+                            config=dict(name='realm'),
+                        ))
         rap = RouterApplicationSession(session, router)
 
         # we never fake out the 'Welcome' message, so there will be no
@@ -346,7 +314,6 @@ class TestBrokerPublish(unittest.TestCase):
         d = txaio.create_future()
 
         class TestSession(ApplicationSession):
-
             def onJoin(self, details):
                 d2 = self.subscribe(lambda: None, 'com.example.topic1')
 
@@ -369,12 +336,14 @@ class TestBrokerPublish(unittest.TestCase):
         ensure a session doesn't get Events if it's closed
         (see also issue #431)
         """
+
         # we want to trigger a deeply-nested condition in
         # processPublish in class Broker -- lets try w/o refactoring
         # anything first...
 
         class TestSession(ApplicationSession):
             pass
+
         session0 = TestSession()
         session1 = TestSession()
         router = mock.MagicMock()
@@ -400,7 +369,8 @@ class TestBrokerPublish(unittest.TestCase):
 
         # here's the main "cheat"; we're faking out the
         # router.authorize because we need it to callback immediately
-        router.authorize = mock.MagicMock(return_value=txaio.create_future_success(dict(allow=True, cache=False, disclose=True)))
+        router.authorize = mock.MagicMock(
+            return_value=txaio.create_future_success(dict(allow=True, cache=False, disclose=True)))
 
         # now we scan call "processPublish" such that we get to the
         # condition we're interested in (this "comes from" session1
@@ -409,20 +379,22 @@ class TestBrokerPublish(unittest.TestCase):
         broker.processPublish(session1, pubmsg)
 
         # neither session should have sent anything on its transport
-        self.assertEquals(session0._transport.method_calls, [])
-        self.assertEquals(session1._transport.method_calls, [])
+        self.assertEqual(session0._transport.method_calls, [])
+        self.assertEqual(session1._transport.method_calls, [])
 
     def test_publish_traced_events(self):
         """
         with two subscribers and message tracing the last event should
         have a magic flag
         """
+
         # we want to trigger a deeply-nested condition in
         # processPublish in class Broker -- lets try w/o refactoring
         # anything first...
 
         class TestSession(ApplicationSession):
             pass
+
         session0 = TestSession()
         session1 = TestSession()
         session2 = TestSession()
@@ -454,7 +426,8 @@ class TestBrokerPublish(unittest.TestCase):
 
         # here's the main "cheat"; we're faking out the
         # router.authorize because we need it to callback immediately
-        router.authorize = mock.MagicMock(return_value=txaio.create_future_success(dict(allow=True, cache=False, disclose=True)))
+        router.authorize = mock.MagicMock(
+            return_value=txaio.create_future_success(dict(allow=True, cache=False, disclose=True)))
 
         # now we scan call "processPublish" such that we get to the
         # condition we're interested in (this "comes from" session1
@@ -463,11 +436,7 @@ class TestBrokerPublish(unittest.TestCase):
         broker.processPublish(session2, pubmsg)
 
         # extract all the event calls
-        events = [
-            call[1][1]
-            for call in router.send.mock_calls
-            if call[1][0] in [session0, session1, session2]
-        ]
+        events = [call[1][1] for call in router.send.mock_calls if call[1][0] in [session0, session1, session2]]
 
         self.assertEqual(2, len(events))
         self.assertFalse(events[0].correlation_is_last)
@@ -478,12 +447,14 @@ class TestBrokerPublish(unittest.TestCase):
         with two subscribers and message tracing the last event should
         have a magic flag
         """
+
         # we want to trigger a deeply-nested condition in
         # processPublish in class Broker -- lets try w/o refactoring
         # anything first...
 
         class TestSession(ApplicationSession):
             pass
+
         session0 = TestSession()
         session1 = TestSession()
         session2 = TestSession()
@@ -521,7 +492,8 @@ class TestBrokerPublish(unittest.TestCase):
 
             # here's the main "cheat"; we're faking out the
             # router.authorize because we need it to callback immediately
-            router.authorize = mock.MagicMock(return_value=txaio.create_future_success(dict(allow=True, cache=False, disclose=True)))
+            router.authorize = mock.MagicMock(
+                return_value=txaio.create_future_success(dict(allow=True, cache=False, disclose=True)))
 
             # now we scan call "processPublish" such that we get to the
             # condition we're interested in; should go to all sessions
@@ -533,8 +505,7 @@ class TestBrokerPublish(unittest.TestCase):
 
             # extract all the event calls
             events = [
-                call[1][1]
-                for call in router.send.mock_calls
+                call[1][1] for call in router.send.mock_calls
                 if call[1][0] in [session0, session1, session2, session3, session4]
             ]
 
@@ -568,7 +539,6 @@ class TestBrokerPublish(unittest.TestCase):
                 self._events.append([args, argv])
 
         class TestSession(ApplicationSession):
-
             def __init__(self, *args, **kw):
                 super().__init__(*args, **kw)
                 self._service_session = Session()
@@ -588,7 +558,8 @@ class TestBrokerPublish(unittest.TestCase):
 
                 subscriptions = []
                 for obj in list(self._service_session._private):
-                    subscription = message.Unsubscribe(self._service_session._session_id, subscription=obj.subscription)
+                    subscription = message.Unsubscribe(self._service_session._session_id,
+                                                       subscription=obj.subscription)
                     router._broker.processUnsubscribe(self._service_session, subscription)
                     subscriptions.append(obj.subscription)
 
@@ -601,22 +572,26 @@ class TestBrokerPublish(unittest.TestCase):
 
                     for args, argv in self._service_session._events:
                         if args[0] == 'wamp.subscription.on_create':
-                            test.assertEqual(args[1], self._service_session._session_id, 'on_create: session id is incorrect!')
+                            test.assertEqual(args[1], self._service_session._session_id,
+                                             'on_create: session id is incorrect!')
                             test.assertTrue(args[2]['id'] in created, 'on_create: subscription id is incorrect!')
                             created.remove(args[2]['id'])
 
                         if args[0] == 'wamp.subscription.on_subscribe':
-                            test.assertEqual(args[1], self._service_session._session_id, 'on_subscribe: session id is incorrect!')
+                            test.assertEqual(args[1], self._service_session._session_id,
+                                             'on_subscribe: session id is incorrect!')
                             test.assertTrue(args[2] in subscribes, 'on_subscribe: subscription id is incorrect!')
                             subscribes.remove(args[2])
 
                         if args[0] == 'wamp.subscription.on_unsubscribe':
-                            test.assertEqual(args[1], self._service_session._session_id, 'on_unsubscribe: session id is incorrect!')
+                            test.assertEqual(args[1], self._service_session._session_id,
+                                             'on_unsubscribe: session id is incorrect!')
                             test.assertTrue(args[2] in unsubscribes, 'on_unsubscribe: subscription id is incorrect!')
                             unsubscribes.remove(args[2])
 
                         if args[0] == 'wamp.subscription.on_delete':
-                            test.assertEqual(args[1], self._service_session._session_id, 'on_delete: session id is incorrect!')
+                            test.assertEqual(args[1], self._service_session._session_id,
+                                             'on_delete: session id is incorrect!')
                             test.assertTrue(args[2] in deletes, 'on_delete: subscription id is incorrect!')
                             deletes.remove(args[2])
 
@@ -657,7 +632,6 @@ class TestBrokerPublish(unittest.TestCase):
                 self._events.append([args, argv])
 
         class TestSession(ApplicationSession):
-
             def __init__(self, *args, **kw):
                 super().__init__(*args, **kw)
                 self._service_session = Session()
@@ -696,22 +670,26 @@ class TestBrokerPublish(unittest.TestCase):
                     for args, argv in self._service_session._events:
 
                         if args[0] == 'wamp.subscription.on_create':
-                            test.assertEqual(args[1], self._service_session._session_id, 'on_create: session id is incorrect!')
+                            test.assertEqual(args[1], self._service_session._session_id,
+                                             'on_create: session id is incorrect!')
                             test.assertTrue(args[2]['id'] in created, 'on_create: subscription id is incorrect!')
                             created.remove(args[2]['id'])
 
                         if args[0] == 'wamp.subscription.on_subscribe':
-                            test.assertEqual(args[1], self._service_session._session_id, 'on_subscribe: session id is incorrect!')
+                            test.assertEqual(args[1], self._service_session._session_id,
+                                             'on_subscribe: session id is incorrect!')
                             test.assertTrue(args[2] in subscribes, 'on_subscribe: subscription id is incorrect!')
                             subscribes.remove(args[2])
 
                         if args[0] == 'wamp.subscription.on_unsubscribe':
-                            test.assertEqual(args[1], self._service_session._session_id, 'on_unsubscribe: session id is incorrect!')
+                            test.assertEqual(args[1], self._service_session._session_id,
+                                             'on_unsubscribe: session id is incorrect!')
                             test.assertTrue(args[2] in unsubscribes, 'on_unsubscribe: subscription id is incorrect!')
                             unsubscribes.remove(args[2])
 
                         if args[0] == 'wamp.subscription.on_delete':
-                            test.assertEqual(args[1], self._service_session._session_id, 'on_delete: session id is incorrect!')
+                            test.assertEqual(args[1], self._service_session._session_id,
+                                             'on_delete: session id is incorrect!')
                             test.assertTrue(args[2] in deletes, 'on_delete: subscription id is incorrect!')
                             deletes.remove(args[2])
 
@@ -730,7 +708,6 @@ class TestRouterSession(unittest.TestCase):
     """
     Tests for crossbar.router.session.RouterSession
     """
-
     def test_wamp_session_on_leave(self):
         """
         wamp.session.on_leave receives valid session_id
