@@ -1,30 +1,7 @@
 #####################################################################################
 #
 #  Copyright (c) Crossbar.io Technologies GmbH
-#
-#  Unless a separate license agreement exists between you and Crossbar.io GmbH (e.g.
-#  you have purchased a commercial license), the license terms below apply.
-#
-#  Should you enter into a separate license agreement after having received a copy of
-#  this software, then the terms of such license agreement replace the terms below at
-#  the time at which such license agreement becomes effective.
-#
-#  In case a separate license agreement ends, and such agreement ends without being
-#  replaced by another separate license agreement, the license terms below apply
-#  from the time at which said agreement ends.
-#
-#  LICENSE TERMS
-#
-#  This program is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License, version 3, as published by the
-#  Free Software Foundation. This program is distributed in the hope that it will be
-#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-#
-#  See the GNU Affero General Public License Version 3 for more details.
-#
-#  You should have received a copy of the GNU Affero General Public license along
-#  with this program. If not, see <http://www.gnu.org/licenses/agpl-3.0.en.html>.
+#  SPDX-License-Identifier: EUPL-1.2
 #
 #####################################################################################
 from crossbar.worker.transport import TransportController
@@ -33,6 +10,7 @@ from twisted.internet.defer import Deferred, DeferredList, maybeDeferred, return
 from twisted.internet.defer import inlineCallbacks
 from twisted.python.failure import Failure
 from twisted.internet.defer import succeed
+from uuid import uuid4
 
 from autobahn import wamp
 from autobahn.util import utcstr
@@ -49,8 +27,7 @@ from crossbar.worker import _appsession_loader
 from crossbar.worker.controller import WorkerController
 from crossbar.worker.rlink import RLinkConfig
 
-
-__all__ = ('RouterController',)
+__all__ = ('RouterController', )
 
 
 class RouterController(TransportController):
@@ -89,16 +66,14 @@ class RouterController(TransportController):
         self.components = {}
 
         # "global" shared between all components
-        self.components_shared = {
-            'reactor': reactor
-        }
+        self.components_shared = {'reactor': reactor}
 
         # map: transport ID -> RouterTransport
         self.transports = {}
 
     def realm_by_name(self, name):
         realm_id = self.realm_to_id.get(name, None)
-        assert(realm_id in self.realms)
+        assert (realm_id in self.realms)
         return self.realms[realm_id]
 
     @property
@@ -139,8 +114,7 @@ class RouterController(TransportController):
         # WorkerController.publish_ready()
         self.publish_ready()
 
-        self.log.info('Router worker session for "{worker_id}" ready',
-                      worker_id=self._worker_id)
+        self.log.info('Router worker session for "{worker_id}" ready', worker_id=self._worker_id)
 
     def onLeave(self, details):
         # when this router is shutting down, we disconnect all our
@@ -158,6 +132,7 @@ class RouterController(TransportController):
                             id=component.id,
                         )
                         component.session.disconnect()
+
                     d.addCallback(done)
                     leaves.append(d)
         dl = DeferredList(leaves, consumeErrors=True)
@@ -217,7 +192,8 @@ class RouterController(TransportController):
         :rtype: dict
         """
         self.log.debug('{klass}.get_router_realm_by_name(realm_name="{realm_name}")',
-                       klass=self.__class__.__name__, realm_name=realm_name)
+                       klass=self.__class__.__name__,
+                       realm_name=realm_name)
 
         if realm_name not in self.realm_to_id:
             raise ApplicationError('crossbar.error.no_such_object', 'No realm with name "{}"'.format(realm_name))
@@ -235,7 +211,9 @@ class RouterController(TransportController):
         :returns: realm statistics object
         :rtype: dict
         """
-        self.log.debug("{name}.get_router_realm_stats(realm_id={realm_id})", name=self.__class__.__name__, realm_id=realm_id)
+        self.log.debug("{name}.get_router_realm_stats(realm_id={realm_id})",
+                       name=self.__class__.__name__,
+                       realm_id=realm_id)
 
         if realm_id is not None and realm_id not in self.realms:
             raise ApplicationError("crossbar.error.no_such_object", "No realm with ID '{}'".format(realm_id))
@@ -269,7 +247,8 @@ class RouterController(TransportController):
         :type details: :class:`autobahn.wamp.types.CallDetails`
         """
         self.log.info('Starting router realm {realm_id} {method}',
-                      realm_id=hlid(realm_id), method=hltype(RouterController.start_router_realm))
+                      realm_id=hlid(realm_id),
+                      method=hltype(RouterController.start_router_realm))
 
         # prohibit starting a realm twice
         #
@@ -299,7 +278,8 @@ class RouterController(TransportController):
         bridge_meta_api = options.get('bridge_meta_api', False)
         if bridge_meta_api:
             # FIXME
-            bridge_meta_api_prefix = 'crossbar.worker.{worker_id}.realm.{realm_id}.root.'.format(worker_id=self._worker_id, realm_id=realm_id)
+            bridge_meta_api_prefix = 'crossbar.worker.{worker_id}.realm.{realm_id}.root.'.format(
+                worker_id=self._worker_id, realm_id=realm_id)
         else:
             bridge_meta_api_prefix = None
 
@@ -341,8 +321,11 @@ class RouterController(TransportController):
 
         yield extra['onready']
         self.set_service_session(rlm.session, realm_name, authrole=svc_authrole)
-        self.log.info('RouterServiceAgent started on realm="{realm_name}" with authrole="{authrole}", authid="{authid}"',
-                      realm_name=realm_name, authrole=svc_authrole, authid=svc_authid)
+        self.log.info(
+            'RouterServiceAgent started on realm="{realm_name}" with authrole="{authrole}", authid="{authid}"',
+            realm_name=realm_name,
+            authrole=svc_authrole,
+            authid=svc_authid)
 
         self.publish('{}.on_realm_started'.format(self._uri_prefix), realm_id)
 
@@ -351,8 +334,11 @@ class RouterController(TransportController):
         caller = details.caller if details else None
         self.publish(topic, event, options=PublishOptions(exclude=caller))
 
-        self.log.info('Realm "{realm_id}" (name="{realm_name}", authrole="{authrole}", authid="{authid}") started', realm_id=realm_id,
-                      realm_name=rlm.session._realm, authrole=svc_authrole, authid=svc_authid)
+        self.log.info('Realm "{realm_id}" (name="{realm_name}", authrole="{authrole}", authid="{authid}") started',
+                      realm_id=realm_id,
+                      realm_name=rlm.session._realm,
+                      authrole=svc_authrole,
+                      authid=svc_authid)
         return event
 
     @wamp.register(None)
@@ -388,11 +374,7 @@ class RouterController(TransportController):
         del self.realms[realm_id]
         del self.realm_to_id[realm_name]
 
-        realm_stopped = {
-            'id': realm_id,
-            'name': realm_name,
-            'detached_sessions': sorted(detached_sessions)
-        }
+        realm_stopped = {'id': realm_id, 'name': realm_name, 'detached_sessions': sorted(detached_sessions)}
 
         self.publish('{}.on_realm_stopped'.format(self._uri_prefix), realm_id)
         returnValue(realm_stopped)
@@ -408,8 +390,10 @@ class RouterController(TransportController):
         :rtype: bool
         """
         result = realm in self.realm_to_id and self.realm_to_id[realm] in self.realms
-        self.log.debug('{func}(realm="{realm}") -> {result}', func=hltype(RouterController.has_realm),
-                       realm=hlid(realm), result=hlval(result))
+        self.log.debug('{func}(realm="{realm}") -> {result}',
+                       func=hltype(RouterController.has_realm),
+                       realm=hlid(realm),
+                       result=hlval(result))
         return result
 
     def has_role(self, realm: str, authrole: str) -> bool:
@@ -429,13 +413,16 @@ class RouterController(TransportController):
         result = realm in self.realm_to_id and self.realm_to_id[realm] in self.realms
         if result:
             realm_id = self.realm_to_id[realm]
-            result = (authrole in self.realms[realm_id].role_to_id and self.realms[realm_id].role_to_id[authrole] in self.realms[realm_id].roles)
+            result = (authrole in self.realms[realm_id].role_to_id
+                      and self.realms[realm_id].role_to_id[authrole] in self.realms[realm_id].roles)
 
             # note: this is to enable eg built-in "trusted" authrole
             result = result or authrole in self._service_sessions[realm]
 
         self.log.debug('{func}(realm="{realm}", authrole="{authrole}") -> {result}',
-                       func=hltype(RouterController.has_role), realm=hlid(realm), authrole=hlid(authrole),
+                       func=hltype(RouterController.has_role),
+                       realm=hlid(realm),
+                       authrole=hlid(authrole),
                        result=hlval(result))
         return result
 
@@ -445,8 +432,10 @@ class RouterController(TransportController):
             self._service_sessions[realm] = {}
         self._service_sessions[realm][authrole] = session
         self.log.info('{func}(session={session}, realm="{realm}", authrole="{authrole}")',
-                      func=hltype(self.set_service_session), session=session,
-                      realm=hlid(realm), authrole=hlid(authrole))
+                      func=hltype(self.set_service_session),
+                      session=session,
+                      realm=hlid(realm),
+                      authrole=hlid(authrole))
 
     def get_service_session(self, realm, authrole):
         authrole = authrole or 'trusted'
@@ -455,8 +444,10 @@ class RouterController(TransportController):
             if authrole in self._service_sessions[realm]:
                 session = self._service_sessions[realm][authrole]
         self.log.debug('{func}(realm="{realm}", authrole="{authrole}") -> {session}',
-                       func=hltype(self.get_service_session), session=session,
-                       realm=hlid(realm), authrole=hlid(authrole))
+                       func=hltype(self.get_service_session),
+                       session=session,
+                       realm=hlid(realm),
+                       authrole=hlid(authrole))
         return succeed(session)
 
     @wamp.register(None)
@@ -498,13 +489,16 @@ class RouterController(TransportController):
         :rtype: dict
         """
         self.log.debug("{name}.get_router_realm_role(realm_id={realm_id}, role_id={role_id})",
-                       name=self.__class__.__name__, realm_id=realm_id, role_id=role_id)
+                       name=self.__class__.__name__,
+                       realm_id=realm_id,
+                       role_id=role_id)
 
         if realm_id not in self.realms:
             raise ApplicationError("crossbar.error.no_such_object", "No realm with ID '{}'".format(realm_id))
 
         if role_id not in self.realms[realm_id].roles:
-            raise ApplicationError("crossbar.error.no_such_object", "No role with ID '{}' on realm '{}'".format(role_id, realm_id))
+            raise ApplicationError("crossbar.error.no_such_object",
+                                   "No role with ID '{}' on realm '{}'".format(role_id, realm_id))
 
         return self.realms[realm_id].roles[role_id].marshal()
 
@@ -526,20 +520,26 @@ class RouterController(TransportController):
         :type details: :class:`autobahn.wamp.types.CallDetails`
         """
         self.log.debug('Starting role "{role_id}" on realm "{realm_id}" {method}',
-                       role_id=role_id, realm_id=realm_id, method=hltype(self.start_router_realm_role))
+                       role_id=role_id,
+                       realm_id=realm_id,
+                       method=hltype(self.start_router_realm_role))
 
         if realm_id not in self.realms:
             raise ApplicationError("crossbar.error.no_such_object", "No realm with ID '{}'".format(realm_id))
 
         if role_id in self.realms[realm_id].roles:
-            raise ApplicationError("crossbar.error.already_exists", "A role with ID '{}' already exists in realm with ID '{}'".format(role_id, realm_id))
+            raise ApplicationError(
+                "crossbar.error.already_exists",
+                "A role with ID '{}' already exists in realm with ID '{}'".format(role_id, realm_id))
 
         realm = self.realms[realm_id].config['name']
         role = RouterRealmRole(role_id, role_config)
         role_name = role.config['name']
 
         if role_name in self.realms[realm_id].role_to_id:
-            raise ApplicationError("crossbar.error.already_exists", "A role with name '{}' already exists in realm with ID '{}'".format(role_name, realm_id))
+            raise ApplicationError(
+                "crossbar.error.already_exists",
+                "A role with name '{}' already exists in realm with ID '{}'".format(role_name, realm_id))
 
         self.realms[realm_id].roles[role_id] = role
         self.realms[realm_id].role_to_id[role_name] = role_id
@@ -550,8 +550,11 @@ class RouterController(TransportController):
         caller = details.caller if details else None
         self.publish(topic, event, options=PublishOptions(exclude=caller))
 
-        self.log.info('Role {role_id} named "{role_name}" started on realm "{realm}"', role_id=hlid(role_id),
-                      role_name=hlid(role_name), realm=hlid(realm), func=hltype(self.start_router_realm_role))
+        self.log.info('Role {role_id} named "{role_name}" started on realm "{realm}"',
+                      role_id=hlid(role_id),
+                      role_name=hlid(role_name),
+                      realm=hlid(realm),
+                      func=hltype(self.start_router_realm_role))
         return event
 
     @wamp.register(None)
@@ -574,7 +577,8 @@ class RouterController(TransportController):
             raise ApplicationError("crossbar.error.no_such_object", "No realm with ID '{}'".format(realm_id))
 
         if role_id not in self.realms[realm_id].roles:
-            raise ApplicationError("crossbar.error.no_such_object", "No role with ID '{}' in realm with ID '{}'".format(role_id, realm_id))
+            raise ApplicationError("crossbar.error.no_such_object",
+                                   "No role with ID '{}' in realm with ID '{}'".format(role_id, realm_id))
 
         role = self.realms[realm_id].roles.pop(role_id)
         del self.realms[realm_id].role_to_id[role.config['name']]
@@ -663,8 +667,7 @@ class RouterController(TransportController):
             self.log.error(emsg)
             raise ApplicationError("crossbar.error.invalid_configuration", emsg)
         else:
-            self.log.debug("Starting {type}-component on router.",
-                           type=config['type'])
+            self.log.debug("Starting {type}-component on router.", type=config['type'])
 
         # resolve references to other entities
         #
@@ -738,11 +741,9 @@ class RouterController(TransportController):
 
             # any exception spilling out from user code in onXXX handlers is fatal!
             def panic(fail, msg):
-                self.log.error(
-                    "Fatal error in component: {msg} - {log_failure.value}",
-                    msg=msg, log_failure=fail
-                )
+                self.log.error("Fatal error in component: {msg} - {log_failure.value}", msg=msg, log_failure=fail)
                 session.disconnect()
+
             session._swallow_error = panic
         except Exception:
             self.log.error(
@@ -809,7 +810,10 @@ class RouterController(TransportController):
 
         self.components[id] = RouterComponent(id, config, session)
         router = self._router_factory.get(realm)
-        self._router_session_factory.add(session, router, authrole=config.get('role', 'anonymous'))
+        self._router_session_factory.add(session,
+                                         router,
+                                         authrole=config.get('role', 'anonymous'),
+                                         authid=uuid4().__str__())
         self.log.debug(
             "Added component {id} (type '{name}')",
             id=id,
@@ -899,11 +903,13 @@ class RouterController(TransportController):
         :type details: :class:`autobahn.wamp.types.CallDetails`
         """
         self.log.info('Starting router transport "{transport_id}" {method}',
-                      transport_id=transport_id, method=hltype(self.start_router_transport))
+                      transport_id=transport_id,
+                      method=hltype(self.start_router_transport))
 
         # prohibit starting a transport twice
         if transport_id in self.transports:
-            _emsg = 'Could not start transport: a transport with ID "{}" is already running (or starting)'.format(transport_id)
+            _emsg = 'Could not start transport: a transport with ID "{}" is already running (or starting)'.format(
+                transport_id)
             self.log.error(_emsg)
             raise ApplicationError('crossbar.error.already_running', _emsg)
 
@@ -911,9 +917,7 @@ class RouterController(TransportController):
         router_transport = self.personality.create_router_transport(self, transport_id, config)
 
         caller = details.caller if details else None
-        event = {
-            'id': transport_id
-        }
+        event = {'id': transport_id}
         topic = '{}.on_router_transport_starting'.format(self._uri_prefix)
         self.publish(topic, event, options=PublishOptions(exclude=caller))
 
@@ -969,8 +973,10 @@ class RouterController(TransportController):
         """
         self.log.debug("{name}.stop_router_transport", name=self.__class__.__name__)
 
-        if transport_id not in self.transports or self.transports[transport_id].state != self.personality.RouterTransport.STATE_STARTED:
-            emsg = "Cannot stop transport: no transport with ID '{}' or transport is already stopping".format(transport_id)
+        if transport_id not in self.transports or self.transports[
+                transport_id].state != self.personality.RouterTransport.STATE_STARTED:
+            emsg = "Cannot stop transport: no transport with ID '{}' or transport is already stopping".format(
+                transport_id)
             self.log.error(emsg)
             raise ApplicationError('crossbar.error.not_running', emsg)
 
@@ -1006,7 +1012,8 @@ class RouterController(TransportController):
     @wamp.register(None)
     def kill_by_authid(self, realm_id, authid, reason, message=None, details=None):
         self.log.info('Killing sessions by authid="{authid}" ..',
-                      realm_id=hlid(realm_id), authid=hlid(authid),
+                      realm_id=hlid(realm_id),
+                      authid=hlid(authid),
                       method=hltype(RouterController.start_router_realm))
 
         if realm_id not in self.realms:
@@ -1029,10 +1036,9 @@ class RouterController(TransportController):
         assert type(realm_id) == str
         assert isinstance(details, CallDetails)
 
-        self.log.info(
-            '{method} Getting router links for realm {realm_id}',
-            realm_id=hlid(realm_id),
-            method=hltype(RouterController.get_router_realm_links))
+        self.log.info('{method} Getting router links for realm {realm_id}',
+                      realm_id=hlid(realm_id),
+                      method=hltype(RouterController.get_router_realm_links))
 
         if realm_id not in self.realms:
             raise ApplicationError("crossbar.error.no_such_object", "No realm with ID '{}'".format(realm_id))
@@ -1059,11 +1065,10 @@ class RouterController(TransportController):
         assert type(link_id) == str
         assert isinstance(details, CallDetails)
 
-        self.log.info(
-            '{method} Get router link {link_id} on realm {realm_id}',
-            link_id=hlid(link_id),
-            realm_id=hlid(realm_id),
-            method=hltype(RouterController.get_router_realm_link))
+        self.log.info('{method} Get router link {link_id} on realm {realm_id}',
+                      link_id=hlid(link_id),
+                      realm_id=hlid(realm_id),
+                      method=hltype(RouterController.get_router_realm_link))
 
         if realm_id not in self.realms:
             raise ApplicationError("crossbar.error.no_such_object", "No realm with ID '{}'".format(realm_id))
@@ -1118,11 +1123,10 @@ class RouterController(TransportController):
         assert type(link_config) == dict
         assert isinstance(details, CallDetails)
 
-        self.log.info(
-            '{method} Router link {link_id} starting on realm {realm_id} ..',
-            link_id=hlid(link_id),
-            realm_id=hlid(realm_id),
-            method=hltype(RouterController.start_router_realm_link))
+        self.log.info('{method} Router link {link_id} starting on realm {realm_id} ..',
+                      link_id=hlid(link_id),
+                      realm_id=hlid(realm_id),
+                      method=hltype(RouterController.start_router_realm_link))
 
         try:
             if realm_id not in self.realms:
@@ -1148,7 +1152,8 @@ class RouterController(TransportController):
             self.publish('{}.on_router_realm_link_started'.format(self._uri_prefix), started)
 
             self.log.info('Router link {link_id} started on realm {realm_id}',
-                          link_id=hlid(link_id), realm_id=hlid(realm_id))
+                          link_id=hlid(link_id),
+                          realm_id=hlid(realm_id))
 
             returnValue(started)
 
@@ -1171,11 +1176,10 @@ class RouterController(TransportController):
         assert type(link_id) == str
         assert isinstance(details, CallDetails)
 
-        self.log.info(
-            '{method} Router link {link_id} stopping on realm {realm_id}',
-            link_id=hlid(link_id),
-            realm_id=hlid(realm_id),
-            method=hltype(RouterController.stop_router_realm_link))
+        self.log.info('{method} Router link {link_id} stopping on realm {realm_id}',
+                      link_id=hlid(link_id),
+                      realm_id=hlid(realm_id),
+                      method=hltype(RouterController.stop_router_realm_link))
 
         if realm_id not in self.realms:
             raise ApplicationError('crossbar.error.no_such_object', 'no realm with ID {}'.format(realm_id))
@@ -1183,8 +1187,7 @@ class RouterController(TransportController):
         rlink_manager = self.realms[realm_id].rlink_manager
 
         if link_id not in self.rlink_manager:
-            raise ApplicationError('crossbar.error.no_such_object',
-                                   'no router link with ID {}'.format(link_id))
+            raise ApplicationError('crossbar.error.no_such_object', 'no router link with ID {}'.format(link_id))
 
         caller = SessionIdent.from_calldetails(details)
 
