@@ -1,30 +1,7 @@
 #####################################################################################
 #
 #  Copyright (c) Crossbar.io Technologies GmbH
-#
-#  Unless a separate license agreement exists between you and Crossbar.io GmbH (e.g.
-#  you have purchased a commercial license), the license terms below apply.
-#
-#  Should you enter into a separate license agreement after having received a copy of
-#  this software, then the terms of such license agreement replace the terms below at
-#  the time at which such license agreement becomes effective.
-#
-#  In case a separate license agreement ends, and such agreement ends without being
-#  replaced by another separate license agreement, the license terms below apply
-#  from the time at which said agreement ends.
-#
-#  LICENSE TERMS
-#
-#  This program is free software: you can redistribute it and/or modify it under the
-#  terms of the GNU Affero General Public License, version 3, as published by the
-#  Free Software Foundation. This program is distributed in the hope that it will be
-#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
-#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-#
-#  See the GNU Affero General Public License Version 3 for more details.
-#
-#  You should have received a copy of the GNU Affero General Public license along
-#  with this program. If not, see <http://www.gnu.org/licenses/agpl-3.0.en.html>.
+#  SPDX-License-Identifier: EUPL-1.2
 #
 #####################################################################################
 
@@ -55,7 +32,6 @@ except (ImportError, SyntaxError):
 else:
     WSGI_TESTS = False
 
-
 WSGI_TESTS = True
 
 
@@ -81,19 +57,17 @@ class FakeWAMPTransport(object):
 
         if isinstance(message, Hello):
             self._session.onMessage(
-                Welcome(1, {"broker": RoleBrokerFeatures(),
-                            "dealer": RoleDealerFeatures()},
-                        authrole="anonymous"))
+                Welcome(1, {
+                    "broker": RoleBrokerFeatures(),
+                    "dealer": RoleDealerFeatures()
+                }, authrole="anonymous"))
         elif isinstance(message, Register):
-            self._session.onMessage(
-                Registered(message.request, message.request))
+            self._session.onMessage(Registered(message.request, message.request))
         elif isinstance(message, Publish):
             if message.acknowledge:
-                self._session.onMessage(
-                    Published(message.request, message.request))
+                self._session.onMessage(Published(message.request, message.request))
         elif isinstance(message, Subscribe):
-            self._session.onMessage(
-                Subscribed(message.request, message.request))
+            self._session.onMessage(Subscribed(message.request, message.request))
         else:
             assert False, message
 
@@ -110,8 +84,7 @@ class RouterWorkerSessionTests(TestCase):
         Set up the common component config.
         """
         self.realm = "realm1"
-        config_extras = DottableDict({"worker": "worker1",
-                                      "cbdir": self.mktemp()})
+        config_extras = DottableDict({"worker": "worker1", "cbdir": self.mktemp()})
         self.config = ComponentConfig(self.realm, extra=config_extras)
 
     def test_basic(self):
@@ -141,11 +114,18 @@ class RouterWorkerSessionTests(TestCase):
         r.onOpen(transport)
 
         realm_config = {
-            "name": "realm1",
-            'roles': [{'name': 'anonymous',
-                       'permissions': [{'subscribe': True,
-                                        'register': True, 'call': True,
-                                        'uri': '*', 'publish': True}]}]
+            "name":
+            "realm1",
+            'roles': [{
+                'name': 'anonymous',
+                'permissions': [{
+                    'subscribe': True,
+                    'register': True,
+                    'call': True,
+                    'uri': '*',
+                    'publish': True
+                }]
+            }]
         }
 
         r.start_router_realm("realm1", realm_config)
@@ -163,8 +143,7 @@ class RouterWorkerSessionTests(TestCase):
         r.start_router_component("newcomponent", component_config)
 
         self.assertEqual(len(r.get_router_components()), 1)
-        self.assertEqual(r.get_router_components()[0]["id"],
-                         "newcomponent")
+        self.assertEqual(r.get_router_components()[0]["id"], "newcomponent")
 
         self.assertEqual(len(_), 1)
         _.pop()  # clear this global state
@@ -184,27 +163,28 @@ class RouterWorkerSessionTests(TestCase):
         r.onOpen(transport)
 
         realm_config = {
-            "name": "realm1",
-            'roles': [{'name': 'anonymous',
-                       'permissions': [{'subscribe': True,
-                                        'register': True, 'call': True,
-                                        'uri': '*', 'publish': True}]}]
+            "name":
+            "realm1",
+            'roles': [{
+                'name': 'anonymous',
+                'permissions': [{
+                    'subscribe': True,
+                    'register': True,
+                    'call': True,
+                    'uri': '*',
+                    'publish': True
+                }]
+            }]
         }
 
         r.start_router_realm("realm1", realm_config)
 
-        component_config = {
-            "type": "class",
-            "classname": "thisisathing.thatdoesnot.exist",
-            "realm": "realm1"
-        }
+        component_config = {"type": "class", "classname": "thisisathing.thatdoesnot.exist", "realm": "realm1"}
 
         with self.assertRaises(ApplicationError) as e:
             r.start_router_component("newcomponent", component_config)
 
-        self.assertIn(
-            "Failed to import class 'thisisathing.thatdoesnot.exist'",
-            str(e.exception.args[0]))
+        self.assertIn("Failed to import class 'thisisathing.thatdoesnot.exist'", str(e.exception.args[0]))
 
         self.assertEqual(len(r.get_router_components()), 0)
 
@@ -221,17 +201,11 @@ class RouterWorkerSessionTests(TestCase):
         transport = FakeWAMPTransport(r)
         r.onOpen(transport)
 
-        realm_config = {
-            "name": "realm1",
-            'roles': []
-        }
+        realm_config = {"name": "realm1", 'roles': []}
 
         r.start_router_realm("realm1", realm_config)
 
-        component_config = {
-            "type": "notathingcrossbarsupports",
-            "realm": "realm1"
-        }
+        component_config = {"type": "notathingcrossbarsupports", "realm": "realm1"}
 
         with self.assertRaises(ApplicationError) as e:
             r.start_router_component("newcomponent", component_config)
@@ -251,10 +225,7 @@ class RouterWorkerSessionTests(TestCase):
         transport = FakeWAMPTransport(r)
         r.onOpen(transport)
 
-        realm_config = {
-            "name": "realm1",
-            'roles': []
-        }
+        realm_config = {"name": "realm1", 'roles': []}
 
         r.start_router_realm("realm1", realm_config)
 
@@ -267,9 +238,7 @@ class RouterWorkerSessionTests(TestCase):
         with self.assertRaises(ApplicationError) as e:
             r.start_router_component("newcomponent", component_config)
 
-        self.assertIn(
-            ("session not derived of ApplicationSession"),
-            str(e.exception.args[0]))
+        self.assertIn(("session not derived of ApplicationSession"), str(e.exception.args[0]))
 
         self.assertEqual(len(r.get_router_components()), 0)
 
@@ -308,10 +277,12 @@ class WebTests(TestCase):
     def setUp(self):
         self.cbdir = self.mktemp()
         os.makedirs(self.cbdir)
-        config_extras = DottableDict({"worker": "worker1",
-                                      "cbdir": self.cbdir.decode('utf8')
-                                      if not isinstance(self.cbdir, str)
-                                      else self.cbdir})
+        config_extras = DottableDict({
+            "worker":
+            "worker1",
+            "cbdir":
+            self.cbdir.decode('utf8') if not isinstance(self.cbdir, str) else self.cbdir
+        })
         self.config = ComponentConfig("realm1", extra=config_extras)
 
     def test_root_not_required(self):
@@ -320,17 +291,13 @@ class WebTests(TestCase):
         will still be routed correctly.
         """
         temp_reactor = SelectReactor()
-        r = router.RouterController(config=self.config,
-                                    reactor=temp_reactor)
+        r = router.RouterController(config=self.config, reactor=temp_reactor)
 
         # Open the transport
         transport = FakeWAMPTransport(r)
         r.onOpen(transport)
 
-        realm_config = {
-            "name": "realm1",
-            'roles': []
-        }
+        realm_config = {"name": "realm1", 'roles': []}
 
         # Make a file
         with open(os.path.join(self.cbdir, 'file.txt'), "wb") as f:
@@ -338,8 +305,7 @@ class WebTests(TestCase):
 
         r.start_router_realm("realm1", realm_config)
         r.start_router_transport(
-            "component1",
-            {
+            "component1", {
                 "type": "web",
                 "endpoint": {
                     "type": "tcp",
@@ -356,8 +322,7 @@ class WebTests(TestCase):
         d1 = treq.get("http://localhost:8080/", reactor=temp_reactor)
         d1.addCallback(lambda resp: self.assertEqual(resp.code, 404))
 
-        d2 = treq.get("http://localhost:8080/static/file.txt",
-                      reactor=temp_reactor)
+        d2 = treq.get("http://localhost:8080/static/file.txt", reactor=temp_reactor)
         d2.addCallback(treq.content)
         d2.addCallback(self.assertEqual, b"hello!")
 
@@ -385,8 +350,7 @@ class WSGITests(TestCase):
     def setUp(self):
         self.cbdir = self.mktemp()
         os.makedirs(self.cbdir)
-        config_extras = DottableDict({"worker": "worker1",
-                                      "cbdir": self.cbdir})
+        config_extras = DottableDict({"worker": "worker1", "cbdir": self.cbdir})
         self.config = ComponentConfig("realm1", extra=config_extras)
 
     def test_basic(self):
@@ -394,22 +358,17 @@ class WSGITests(TestCase):
         A basic WSGI app can be ran.
         """
         temp_reactor = SelectReactor()
-        r = router.RouterController(config=self.config,
-                                    reactor=temp_reactor)
+        r = router.RouterController(config=self.config, reactor=temp_reactor)
 
         # Open the transport
         transport = FakeWAMPTransport(r)
         r.onOpen(transport)
 
-        realm_config = {
-            "name": "realm1",
-            'roles': []
-        }
+        realm_config = {"name": "realm1", 'roles': []}
 
         r.start_router_realm("realm1", realm_config)
         r.start_router_transport(
-            "component1",
-            {
+            "component1", {
                 "type": "web",
                 "endpoint": {
                     "type": "tcp",
@@ -444,22 +403,17 @@ class WSGITests(TestCase):
         A basic WSGI app can be ran, with subresources
         """
         temp_reactor = SelectReactor()
-        r = router.RouterController(config=self.config,
-                                    reactor=temp_reactor)
+        r = router.RouterController(config=self.config, reactor=temp_reactor)
 
         # Open the transport
         transport = FakeWAMPTransport(r)
         r.onOpen(transport)
 
-        realm_config = {
-            "name": "realm1",
-            'roles': []
-        }
+        realm_config = {"name": "realm1", 'roles': []}
 
         r.start_router_realm("realm1", realm_config)
         r.start_router_transport(
-            "component1",
-            {
+            "component1", {
                 "type": "web",
                 "endpoint": {
                     "type": "tcp",
